/*********************************************************************************************************************************
 * Autorskie Prawa Majątkowe - Moose Spółka z ograniczoną odpowiedzialnością
 *
 * Copyright 2017 Moose Spółka z ograniczoną odpowiedzialnością
 ********************************************************************************************************************************/

package eu.mooseinc.dell.form.entity;

import eu.mooseinc.dell.form.entity.base.BaseEntity;
import eu.mooseinc.dell.form.util.ColumnLengthConstants;
import java.util.Date;
import javax.persistence.Column;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

/**
 * Entity model for registration
 */
public class Registration extends BaseEntity {

	/**
	 * Entity name.
	 */
	@NotNull(message = "{notification.entityName.required}")
	@Size(min = 1, max = ColumnLengthConstants.SHORT_TEXT_LENGTH, message = "{notification.entityName.size}")
	private String entityName;

	/**
	 * Name.
	 */
	@NotNull(message = "{notification.name.required}")
	@Size(min = 1, max = ColumnLengthConstants.SHORT_TEXT_LENGTH, message = "${notification.name.size}")
	private String name;

	/**
	 * Surname
	 */
	@NotNull(message = "{notification.surname.required}")
	@Size(min = 1, max = ColumnLengthConstants.SHORT_TEXT_LENGTH, message = "${notification.name.size}")
	private String surname;

	/**
	 * E-mail address.
	 */
	@NotNull(message = "{notification.email.required}")
	@Size(min = 1, max = ColumnLengthConstants.EMAIL_LENGTH, message = "{notification.email.size}")
	private String email;

	/**
	 * Type of membership.
	 */
	@NotNull(message = "{notification.marketSegment.required}")
	private MembershipType membershipType;

	/**
	 * Message body.
	 */
	@Size(max = ColumnLengthConstants.STANDARD_TEXT_LENGTH, message = "{notification.message.size}")
	private String message;

	/**
	 * Registration date.
	 */
	@Column(name = "registration_date", nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	private Date registrationDate;

	/**
	 * Constructor.
	 */
	public Registration() {
	}

	/**
	 * Returns entity name.
	 *
	 * @return entity name.
	 */
	public String getEntityName() {
		return entityName;
	}

	/**
	 * Set entity name.
	 *
	 * @param entityName entity name.
	 */
	public void setEntityName(String entityName) {
		this.entityName = entityName;
	}

	/**
	 * Returns name.
	 *
	 * @return name.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Set name.
	 *
	 * @param name name.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Returns surname.
	 *
	 * @return surname.
	 */
	public String getSurname() {
		return surname;
	}

	/**
	 * Set surname.
	 *
	 * @param surname surname.
	 */
	public void setSurname(String surname) {
		this.surname = surname;
	}

	/**
	 * Returns e-mail.
	 *
	 * @return e-mail.
	 */
	public String getEmail() {
		return email;
	}

	/**
	 * Set e-mail.
	 *
	 * @param email e-mail.
	 */
	public void setEmail(String email) {
		this.email = email;
	}

	/**
	 * Returns membership type
	 *
	 * @return membership type.
	 */
	public MembershipType getMembershipType() {
		return membershipType;
	}

	/**
	 * Set membership type.
	 *
	 * @param membershipType Membership Type
	 */
	public void setMembershipType(MembershipType membershipType) {
		this.membershipType = membershipType;
	}


	/**
	 * Returns message body.
	 *
	 * @return message body.
	 */
	public String getMessage() {
		return message;
	}

	/**
	 * Set message body.
	 *
	 * @param message message body.
	 */
	public void setMessage(String message) {
		this.message = message;
	}

	/**
	 * Set registration date.
	 *
	 * @param registrationDate registration date.
	 */
	public void setRegistrationDate(Date registrationDate) {
		this.registrationDate = registrationDate;
	}

	/**
	 * Returns registration date.
	 *
	 * @return registration date.
	 */
	public Date getRegistrationDate() {
		return registrationDate;
	}

}
