/*********************************************************************************************************************************
 * Autorskie Prawa Majątkowe - Moose Spółka z ograniczoną odpowiedzialnością
 *
 * Copyright 2017 Moose Spółka z ograniczoną odpowiedzialnością
 ********************************************************************************************************************************/

package eu.mooseinc.dell.form.util;

import com.google.common.base.Preconditions;
import java.io.File;
import java.io.IOException;
import java.nio.file.FileSystems;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;

/**
 * Klasa pomocnicza dla plików.
 */
public class FileFormUtils {
	/**
	 * Konstruktor prywatny.
	 */
	private FileFormUtils() {
	}

	/**
	 * Pobiera ścieżkę katalogu tymczasowego sesji.
	 *
	 * @param temporaryDirectory katalog tymczasowy.
	 * @param sessionId identyfikator sesji.
	 *
	 * @return ścieżka katalogu tymczasowego.
	 */
	public static String getSessionDirectoryName(final String temporaryDirectory, final String sessionId) {
		Preconditions.checkArgument(StringUtils.isNotEmpty(temporaryDirectory));
		Preconditions.checkArgument(StringUtils.isNotEmpty(sessionId));

		final StringBuilder directoryName = new StringBuilder(temporaryDirectory);
		directoryName.append(FileSystems.getDefault().getSeparator());
		directoryName.append(sessionId);

		return directoryName.toString();
	}


	/**
	 * Czyści katalog tymczasowy sesji.
	 *
	 * @param directoryName katalog tymczasowy.
	 *
	 * @throws IOException jeśli wystąpi błąd podczas czyszczenia katalogu.
	 */
	public static void cleanSessionDirectory(final String directoryName) throws IOException {
		final File directory = new File(directoryName);
		FileUtils.cleanDirectory(directory);
	}

}
